@@ -2,13 +2,14 @@
 
 import socket
 import struct
-import msgpack
 import logging
 from collections import defaultdict
 from hash_ring import HashRing
 
 import gsocketpool
 
+from serializer import MsgpackSerializer
+
 HEADER_SIZE = 24
 
 STRUCT_HEADER = '!BBHBBHLLQ'
@@ -45,11 +46,14 @@ class MemcacheConnection(object):
     :param int timeout: (optional) Timeout.
     :param bool lazy: (optional) If set to True, the socket connection is not
         established until you specifically call :func:`open() <gmemcache.MemcacheConnection.open>`.
+    :param bool serializer: (optional) The class used to serialize the value to be cached.
+        :class:`MsgpackSerializer <gmemcache.MsgpackSerializer>` is used by default.
     """
 
-    def __init__(self, hosts, timeout=5, lazy=False):
+    def __init__(self, hosts, timeout=5, lazy=False, serializer=MsgpackSerializer):
         self._hosts = hosts
         self._timeout = timeout
+        self._serializer = serializer()
 
         self._sockets = None
         self._ring = HashRing(self._hosts)
@@ -58,11 +62,15 @@ class MemcacheConnection(object):
             self.open()
 
     def open(self):
+        """Opens a connection."""
+
         self._sockets = {}
         for host in self._hosts:
             self._sockets[host] = self._connect(host)
 
     def close(self):
+        """Closes the connection."""
+
         try:
             for sock in self._sockets.values():
                 sock.close()
@@ -80,12 +88,22 @@ class MemcacheConnection(object):
         return sock
 
     def is_connected(self):
+        """Returns whether the connection has already been established.
+
+        :rtype: bool
+        """
+
         if self._sockets:
             return True
         else:
             return False
 
     def reconnect(self, host):
+        """Reconnects the existing connection to the specified host.
+
+        :param str host: The target host
+        """
+
         try:
             self._sockets[host].close()
         except:
@@ -97,6 +115,11 @@ class MemcacheConnection(object):
             logging.exception('Failed to connect to the memcached: %s' % (host,))
 
     def get(self, key):
+        """Retreives a record with the specified key.
+
+        :param str key: Cache key.
+        """
+
         assert self._sockets is not None, 'The connection has not been created'
 
         host = self._ring.get_node(key)
@@ -116,12 +139,17 @@ class MemcacheConnection(object):
         (_, value) = struct.unpack('!L%ds' % (resp['bodylen'] - 4,), resp['content'])
 
         if resp['status'] == STATUS_SUCCESS:
-            return self._unpack(value)
+            return self._deserialize(value)
 
         else:
             return None
 
     def get_multi(self, keys):
+        """Retreives multiple records with the specified keys.
+
+        :param list keys: Cache keys.
+        """
+
         assert self._sockets is not None, 'The connection has not been created'
 
         if not keys:
@@ -176,16 +204,23 @@ class MemcacheConnection(object):
                 if resp['status'] == STATUS_SUCCESS:
                     (_, key, value) = struct.unpack('!L%ds%ds' % (resp['keylen'], resp['bodylen'] - resp['keylen'] - 4),
                                                     resp['content'])
-                    ret[key] = self._unpack(value)
+                    ret[key] = self._deserialize(value)
 
         return ret
 
     def set(self, key, value, lifetime=0):
+        """Saves a record to the cache.
+
+        :param str key: Cache key.
+        :param value: Value to be cached.
+        :param int lifetime: The number of seconds until the records will expire.
+        """
+
         assert self._sockets is not None, 'The connection has not been created'
 
         host = self._ring.get_node(key)
         sock = self._sockets[host]
-        packed_value = self._pack(value)
+        packed_value = self._serialize(value)
 
         try:
             sock.sendall(struct.pack(STRUCT_SET % (len(key), len(packed_value)),
@@ -207,6 +242,12 @@ class MemcacheConnection(object):
             return False
 
     def set_multi(self, data, lifetime=0):
+        """Saves multiple records to the cache.
+
+        :param dict data: Records to be cached.
+        :param int lifetime: The number of seconds until the records will expire.
+        """
+
         assert self._sockets is not None, 'The connection has not been created'
 
         if not data:
@@ -220,7 +261,7 @@ class MemcacheConnection(object):
         for (host, host_data) in divided_data.iteritems():
             for (n, (key, value)) in enumerate(host_data.iteritems()):
                 host = self._ring.get_node(key)
-                packed_value = self._pack(value)
+                packed_value = self._serialize(value)
                 if n != len(host_data) - 1:
                     msgs[host] += struct.pack(STRUCT_SET % (len(key), len(packed_value)),
                                               REQUEST_MAGIC,
@@ -267,11 +308,17 @@ class MemcacheConnection(object):
 
         return retval
 
-    def _pack(self, value):
-        return msgpack.packb(value, encoding='utf-8')
+    def _serialize(self, value):
+        if self._serializer:
+            return self._serializer.serialize(value)
+        else:
+            return value
 
-    def _unpack(self, value):
-        return msgpack.unpackb(value, encoding='utf-8', use_list=False)
+    def _deserialize(self, value):
+        if self._serializer:
+            return self._serializer.deserialize(value)
+        else:
+            return value
 
     def _get_response(self, sock):
         header = self._read(HEADER_SIZE, sock)
@@ -301,5 +348,8 @@ class MemcacheConnection(object):
 
 
 class MemcachePoolConnection(MemcacheConnection, gsocketpool.Connection):
+    """Memcache connection wrapper class for `gsocketpool <https://github.com/studio-ousia/gsocketpool>`_.
+    """
+
     def __init__(self, hosts, timeout=5):
         MemcacheConnection.__init__(self, hosts, timeout, lazy=False)
